//
//  TableViewController.swift
//  Insert
//
//  Created by Mark Meretzky on 1/13/15.
//  Copyright (c) 2015 Mark Meretzky. All rights reserved.
//

import UIKit;

class TableViewController: UITableViewController, UITextFieldDelegate {
	let cellReuseIdentifier: String = "insert";

	var lines: [String] = [
		"First line",
		"Second line",
		"Third line"
	];

	var changing: NSIndexPath? = nil;	//cell being edited, if any

	//Called by app delegate.

	override init(nibName: String?, bundle: NSBundle?) {
		super.init(nibName: nibName, bundle: bundle);
		setTitle();
	}

	//not called 
	required init(coder aDecoder: NSCoder) {
		super.init(nibName: nil, bundle: nil);
		setTitle();
	}

	func setTitle () {
		let suffix: String = lines.count == 1 ? "" : "s";
		title = "TableView with \(lines.count) Cell\(suffix)";
	}

	override func viewDidLoad() {
		super.viewDidLoad();
		tableView.registerClass(UITableViewCell.self, forCellReuseIdentifier: cellReuseIdentifier);

		// Uncomment the following line to preserve selection between presentations
		// clearsSelectionOnViewWillAppear = false;

		// Uncomment the following line to display an Edit button in the navigation bar for this view controller.
		navigationItem.rightBarButtonItem = editButtonItem();
	}
	
	override func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning();
		// Dispose of any resources that can be recreated.
	}

	// MARK: - Table view data source

	override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
		// Return the number of sections.
		return 1;
	}

	override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
		// Return the number of rows in the section.
		return lines.count;
    }

	override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
		let cell = tableView.dequeueReusableCellWithIdentifier(cellReuseIdentifier, forIndexPath: indexPath) as UITableViewCell

		// Configure the cell...
		cell.textLabel!.text = lines[indexPath.row];
		return cell;
	}

	/*
	// Override to support conditional editing of the table view.
	override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		// Return false if you do not want the specified item to be editable.
        return true;
	}
	*/

	//Give the last (bottommost) cell a green plus.  Give the others red minuses.

	override func tableView(tableView: UITableView, editingStyleForRowAtIndexPath indexPath: NSIndexPath)
		-> UITableViewCellEditingStyle {

		if indexPath.row == lines.count - 1 {
			return UITableViewCellEditingStyle.Insert;
		}
		return UITableViewCellEditingStyle.Delete;
	}

	// Override to support editing the table view.

	override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
		if editingStyle == .Delete {
			// Delete the row from the data source.
			lines.removeAtIndex(indexPath.row);
			tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: UITableViewRowAnimation.Fade);
		} else if editingStyle == .Insert  && changing == nil {
			// Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
			lines.append("New line");
			tableView.reloadData();
			
			//The new cell's textLabel has an all-zero frame.
			//We will copy the frame of an established cell instead.
			let ip: NSIndexPath = NSIndexPath(forRow: 0, inSection: 0);
			let archetype: UITableViewCell? = tableView.cellForRowAtIndexPath(ip);
			assert(archetype != nil);

			//Edit the text of the new cell.
			changing = NSIndexPath(forRow: lines.count - 1, inSection: 0);
			let cell: UITableViewCell? = tableView.cellForRowAtIndexPath(changing!);
			assert(cell != nil);

			let textField: UITextField = UITextField(frame: archetype!.textLabel!.frame);
			textField.frame.origin.y += 0.5;	//kludge
			textField.backgroundColor = cell!.textLabel!.backgroundColor;
			textField.textColor = cell!.textLabel!.textColor;
			textField.font = archetype!.textLabel!.font;
			textField.text = cell!.textLabel!.text;
			textField.delegate = self;
			
			cell!.textLabel!.text = "";
			cell!.contentView.addSubview(textField);
			textField.becomeFirstResponder();
		}
	}

	// Override to support rearranging the table view.

	override func tableView(tableView: UITableView, moveRowAtIndexPath fromIndexPath: NSIndexPath, toIndexPath: NSIndexPath) {
		let line: String = lines[fromIndexPath.row];
		lines.removeAtIndex(fromIndexPath.row);
		lines.insert(line, atIndex: toIndexPath.row);

		//If the green cell has just been dragged up,
		//change it to red and make the bottom cell green.
		tableView.reloadData();
	}

	// Override to support conditional rearranging of the table view.
	override func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		// Return false if you do not want the item to be re-orderable.
		return true;
	}

	/*
	// MARK: - Navigation

	// In a storyboard-based application, you will often want to do a little preparation before navigation
	override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
		// Get the new view controller using [segue destinationViewController].
		// Pass the selected object to the new view controller.
	}
	*/
	
	//Called when return key is pressed, or when user starts editing a different cell.
	//Decide if the app should accept this input and hide the keyboard.
	
	func textFieldShouldReturn(textField: UITextField) -> Bool {
		textField.resignFirstResponder();	//Hide keyboard.
		return true;	//The text field should do its default behavior.
	}
	
	//Called when we're done editing the text in a cell.
	//Process the text that was input.
	
	func textFieldDidEndEditing(textField: UITextField) {
		if changing != nil {
			lines[changing!.row] = textField.text;
			let cell: UITableViewCell? = tableView.cellForRowAtIndexPath(changing!);
			cell!.textLabel!.text = textField.text;
			textField.removeFromSuperview();
			tableView.deselectRowAtIndexPath(changing!, animated: true); //Remove the gray.
			changing = nil;
		}
	}

}
